@@ -4,9 +4,9 @@
 This package contains all refinery units. To write an executable refinery unit,
 it is sufficient to write a class that inherits from `refinery.units.Unit` and
 implements `refinery.units.Unit.process`. If the operation implemented by this
-unit should be reversible, then a method called `reverse` with the same signature
-has to be implemented. For example, the following would be a minimalistic
-approach to implement `refinery.hex`:
+unit should be reversible, then a method called `reverse` with the same
+signature has to be implemented. For example, the following would be a
+minimalistic approach to implement `refinery.hex`:
 
     from refinery import Unit
 
@@ -21,8 +21,8 @@ the script is executed.
 ### Command Line Parameters
 
 If you want your custom refinery unit to accept command line parameters, you can
-write an initialization routine. For example, the following unit implements a very
-simple XOR unit (albeit less versatile than the already existing `refinery.xor`):
+write an initialization routine. For example, the following unit implements a
+very simple XOR unit (less versatile than the already existing `refinery.xor`):
 
     from refinery import Unit, arg
     import itertools
@@ -38,17 +38,17 @@ simple XOR unit (albeit less versatile than the already existing `refinery.xor`)
             return data
 
 The `refinery.arg` decorator is optional and only used here to provide a help
-message on the command line. The example also shows that the `__init__` code can be
-left empty: In this case, refinery automatically adds boilerplate code that copies
-all `__init__` parameters to the `args` member variable of the unit. In this case,
-the constructor will be completed to have the following code:
+message on the command line. The example also shows that the `__init__` code can
+be left empty: In this case, refinery automatically adds boilerplate code that
+copies all `__init__` parameters to the `args` member variable of the unit. In
+this case, the constructor will be completed to have the following code:
 
         def __init__(self, key: arg(help='Encryption key')):
             super().__init__(key=key)
 
-The option of writing an empty `__init__` was added because it is rarely needed to
-perform any processing of the input arguments. The command line help for this unit
-will look as follows:
+The option of writing an empty `__init__` was added because it is rarely needed
+to perform any processing of the input arguments. The command line help for this
+unit will look as follows:
 
     usage: myxor [-h] [-Q] [-0] [-v] key
 
@@ -63,9 +63,9 @@ will look as follows:
 
 ### Refinery Syntax in Code
 
-Refinery units can be used in Python code (and a Python repl) in nearly the same way
-as on the command line. As one example, consider the following unit that can decode
-base64 with a custom alphabet using `refinery.map` and `refinery.b64`:
+Refinery units can be used in Python code (and a Python repl) in nearly the same
+way as on the command line. As one example, consider the following unit that can
+decode base64 with a custom alphabet using `refinery.map` and `refinery.b64`:
 
     from refinery import Unit, b64, map
 
@@ -87,23 +87,69 @@ base64 with a custom alphabet using `refinery.map` and `refinery.b64`:
         def reverse(self, data):
             return data | -b64 | map(self._b64alphabet, self.args.alphabet)
 
-The syntax does not work exactly as on the command line, but it has been designed to
+The syntax does not work exactly as on the command line, but it was designed to
 be as similar as possible:
 
 - The binary or operator `|` can be used to combine units into pipelines.
 - Combining a pipeline from the left with a byte string or io stream object will
-  invoke it, the result of the operation is the final output.
+  feed this byte string into the unit.
 - Unary negation of a reversible unit is equivalent to using the `-R` switch for
   reverse mode.
+- A pipeline is an iterable of output chunks, but there is quite a selection of
+  objects that can be connected to a pipeline from the right using `|` for
+  various different output options. See below for details.
 
-If you want to use frames in code, simply omit any pipe before a square bracked. For
-example, the first example from the `refinery.lib.frame` documentation translates to
-the following Python code:
+If you want to use frames in code, simply omit any pipe before a square bracked.
+For example, the first example from the `refinery.lib.frame` documentation
+translates to the following Python code:
 
-    In [1]: from refinery import *
+    >>> from refinery import *
+    >>> B'OOOOOOOO' | chop(2) [ ccp(B'F') | cca(B'.') ]| ...
+    >>> bytearray(b'FOO.FOO.FOO.FOO.')
 
-    In [2]: B'OOOOOOOO' | chop(2) [ ccp(B'F') | cca(B'.') ]
-    Out[2]: bytearray(b'FOO.FOO.FOO.FOO.')
+In the above example, the pipeline is piped to a literal ellipsis (`...`) to get
+the final result. The following section lists the other output options.
+
+### Output Options in Code
+
+You can connect a pipeline to any binary i/o stream, and the output of the
+pipeline will be written to that stream. Example:
+
+    with open('output', 'wb') as stream:
+        B'BINARY REFINERY' | xor(0x13) | stream
+
+Furthermore, you can connect pipelines to any callable, and you can always use
+a literal ellipsis (`...`) to represent the identity function. The result of
+this is that you receive the raw output from the pipeline:
+
+    >>> B'BINARY REFINERY' | xor(0x13) | ...
+    bytearray(b'QZ]RAJ3AVUZ]VAJ')
+
+You can also connect to sets and lists containing a single callable. In this
+case, the callable will be applied to each output chunk and all results will be
+collected in a list or set, respectively. Examples:
+
+    >>> B'ABABCBABABCHB' | rex('.B') | [str]
+    ['AB', 'AB', 'CB', 'AB', 'AB', 'HB']
+    >>> B'ABABCBABABCHB' | rex('.B') | {str}
+    {'AB', 'CB', 'HB'}
+
+You can also consume into a dictionary in a similar way:
+
+    >>> B'ABABCBABABCHB' | rex('.(?P<k>.)B') | {'k': str}
+    {A: ['BAB', 'BAB'], H: ['CHB']}
+
+Here, the dictionary is expected to contain exactly one key-value pair. The key
+is the name of a meta variable and the value is a conversion function. The
+result will be a dictionary where all converted results have been grouped under
+the respective value of their meta variable. With all of the above options, it
+is always possible to use a literal ellipsis (`...`).
+
+You can connect pipelines to `bytearray` and (writable) `memoryview` instances.
+In this case, the output will be appended to the end of this buffer. Finally, if
+you connect a pipeline to `None`, this will execute the unit but discard all
+output. This is useful for using units with side-effects, like `refinery.peek`,
+in a REPL.
 """
 from __future__ import annotations
 
@@ -149,7 +195,7 @@ from argparse import (
 
 from refinery.lib.argformats import pending, manifest, multibin, number, sliceobj, VariableMissing
 from refinery.lib.argparser import ArgumentParserWithKeywordHooks, ArgparseError
-from refinery.lib.tools import documentation, isstream, lookahead, autoinvoke, skipfirst, isbuffer
+from refinery.lib.tools import documentation, isstream, lookahead, autoinvoke, one, skipfirst, isbuffer
 from refinery.lib.frame import Framed, Chunk
 from refinery.lib.structures import MemoryFile
 
@@ -882,6 +928,19 @@ class LogLevel(IntEnum):
     INFO     = logging.INFO      # noqa
     DEBUG    = logging.DEBUG     # noqa
 
+    @property
+    def verbosity(self) -> int:
+        if self.value >= LogLevel.DETACHED:
+            return -1
+        if self.value >= LogLevel.WARNING:
+            return +0
+        if self.value >= LogLevel.INFO:
+            return +1
+        if self.value >= LogLevel.DEBUG:
+            return +2
+        else:
+            return -1
+
 
 class DelayedArgumentProxy:
     """
@@ -1135,14 +1194,18 @@ class Unit(UnitBase, abstract=True):
     def __iter__(self):
         return self
 
+    @property
+    def leniency(self) -> int:
+        return getattr(self.args, 'lenient', 0)
+
     def _exception_handler(self, exception: BaseException, data: ByteString):
-        if self.args.lenient > 1:
+        if self.leniency > 1:
             try:
                 return exception.partial
             except AttributeError:
                 return data
         if self.log_level >= LogLevel.DETACHED:
-            if isinstance(exception, RefineryPartialResult) and self.args.lenient > 0:
+            if isinstance(exception, RefineryPartialResult) and self.leniency > 0:
                 return None
             raise exception
         elif isinstance(exception, RefineryCriticalException):
@@ -1155,7 +1218,7 @@ class Unit(UnitBase, abstract=True):
             raise exception
         elif isinstance(exception, RefineryPartialResult):
             self.log_warn(F'error, partial result returned: {exception}')
-            if self.args.lenient < 1:
+            if self.leniency < 1:
                 return None
             return exception.partial
         elif isinstance(exception, RefineryImportMissing):
@@ -1310,22 +1373,53 @@ class Unit(UnitBase, abstract=True):
         return result
 
     def __or__(self, stream: Union[BinaryIO, Unit]):
+        def get_converter(it: Iterable):
+            try:
+                c = one(it)
+            except LookupError:
+                return None
+            if ... is c:
+                def identity(x):
+                    return x
+                return identity
+            if callable(c):
+                return c
+
+        if stream is None:
+            with open(os.devnull, 'wb') as null:
+                self | null
+            return
         if isinstance(stream, type) and issubclass(stream, Entry):
             stream = stream()
+        if isinstance(stream, type(...)):
+            def stream(c): return c
         if isinstance(stream, Entry):
             return stream.__copy__().__ror__(self)
         elif isinstance(stream, list):
-            stream.extend(self)
-            return stream
+            converter = get_converter(stream)
+            if converter is None:
+                stream.extend(self)
+                return stream
+            return [converter(chunk) for chunk in self]
         elif isinstance(stream, set):
-            stream.update(self)
-            return stream
+            converter = get_converter(stream)
+            if converter is None:
+                stream.update(self)
+                return stream
+            return {converter(chunk) for chunk in self}
         elif isinstance(stream, dict):
-            if len(stream) == 1:
-                key, check = next(iter(stream.items()))
-                if check is ...:
-                    return {item[key]: item for item in self}
-            raise ValueError('dict consumption target must be of format {"key": ...}')
+            key, convert = one(stream.items())
+            output: Dict[Any, List[Any]] = {}
+            for item in self:
+                try:
+                    value = item.meta[key]
+                except KeyError:
+                    value = None
+                bag = output.setdefault(value, [])
+                if convert is not ...:
+                    item = convert(item)
+                bag.append(item)
+            return output
         elif isinstance(stream, (bytearray, memoryview)):
             with MemoryFile(stream) as stdout:
                 return (self | stdout).getvalue()
