@@ -4,33 +4,83 @@ import re
 
 from fnmatch import fnmatch
 from ipaddress import ip_address
+from typing import AnyStr
 from urllib.parse import urlparse
 from string import ascii_letters
+from pathlib import Path
+from enum import Enum
 
 from refinery.units.pattern import arg, PatternExtractor
 from refinery.units import RefineryCriticalException
 from refinery.lib.patterns import indicators
 
 
+class LetterWeight:
+    def __init__(self, weight):
+        try:
+            self._weights = weight._weights
+        except AttributeError:
+            pass
+        else:
+            return
+        self._weights = {
+            letter: weight for letters, weight in weight.items() for letter in letters
+        }
+        for letter in range(0x100):
+            self._weights.setdefault(letter, 0)
+
+    def __call__(self, data: AnyStr) -> float:
+        if isinstance(data, str):
+            data = data.encode('latin1')
+        return sum(self._weights[c] for c in data) / len(data) / max(self._weights.values())
+
+
+class LetterWeights(LetterWeight, Enum):
+    IOC = LetterWeight({
+        B'^`': 1,
+        B'!$%&()*+-<=>?[]{}~\t': 2,
+        B'ABCDEFGHIJKLMNOPQRSTUVWXYZ': 4,
+        B'.,:;#/\\|@_ ': 5,
+        B'0123456789abcdefghijklmnopqrstuvwxyz': 8,
+    })
+    Path = LetterWeight({
+        B'^`': 1,
+        B'$%&()*+-<=>?[]{}~\t': 2,
+        B'.,:;#/\\|@_ ': 4,
+        B'0123456789': 4,
+        B'ABCDEFGHIJKLMNOPQRSTUVWXYZ': 5,
+        B'abcdefghijklmnopqrstuvwxyz': 8,
+    })
+
+
 class xtp(PatternExtractor):
     """
-    Extract Patterns: Uses regular expressions to extract indicators from the
-    input data and optionally filters these results heuristically. The unit is
-    designed to extract indicators such as domain names and IP addresses, see
-    below for a complete list. To extract data formats such as hex-encoded
-    data, use `refinery.carve`.
+    Extract Patterns: Uses regular expressions to extract indicators from the input data and
+    optionally filters these results heuristically. The unit is designed to extract indicators
+    such as domain names and IP addresses, see below for a complete list. To extract data
+    formats such as hex-encoded data, use `refinery.carve`.
     """
 
     def __init__(
         self,
-        *pattern: arg('pattern', type=str, default=('hostname', 'url', 'email'), help=(
-            'Choose the pattern to extract, defaults are hostname, url, and email. '
-            'Use an asterix character to select all available patterns. The available '
-            'patterns are: {}'.format(', '.join(p.dashname for p in indicators)))),
-        filter: arg('-f', dest='filter', action='count', help=(
-            'If this setting is enabled, the xtp unit will attempt to reduce the number '
-            'of false positives by certain crude heuristics. Specify multiple times to '
-            'make the filtering more aggressive.')) = 0,
+        *pattern: arg('pattern', type=str,
+            default=(
+                indicators.hostname.name,
+                indicators.url.name,
+                indicators.email.name,
+            ), help=(
+                'Choose the pattern to extract. The unit uses {{default}} by default. Use an '
+                'asterix character to select all available patterns. The available patterns '
+                'are: {}'.format(', '.join(p.dashname for p in indicators))
+            )
+        ),
+        filter: arg('-f', dest='filter', action='count',
+            help=(
+                'If this setting is enabled, the xtp unit will attempt to reduce the number '
+                'of false positives by certain crude heuristics. Specify multiple times to '
+                'make the filtering more aggressive.'
+            )
+        ) = 0,
         min=1, max=None, len=None, stripspace=False, duplicates=False, longest=False, take=None
     ):
         self.superinit(super(), **vars(), ascii=True, utf16=True)
@@ -38,10 +88,10 @@ class xtp(PatternExtractor):
         patterns = {
             p for name in pattern for p in indicators if fnmatch(p.dashname, name)
         }
-        if indicators.hostname in patterns:
-            patterns.remove(indicators.hostname)
-            patterns.add(indicators.ipv4)
-            patterns.add(indicators.domain)
+        # if indicators.hostname in patterns:
+        #     patterns.remove(indicators.hostname)
+        #     patterns.add(indicators.ipv4)
+        #     patterns.add(indicators.domain)
         patterns = [F'(?P<{p.name}>{p.value})' for p in patterns]
         if not patterns:
             raise RefineryCriticalException('The given mask does not match any known indicator pattern.')
@@ -131,7 +181,7 @@ class xtp(PatternExtractor):
                 value = value[:pos]
         if not self.args.filter:
             return value
-        if name == 'ipv4':
+        if name == indicators.ipv4.name:
             ocets = [int(x) for x in value.split(B'.')]
             if ocets.count(0) >= 3:
                 return None
@@ -146,9 +196,18 @@ class xtp(PatternExtractor):
             if not ip.is_global:
                 if self.args.filter >= 3 or not ip.is_private:
                     return None
-        elif name in {'url', 'socket', 'hostname', 'domain', 'subdomain'}:
-            if name != 'url' and len(value) > 0x100 and self.args.filter >= 2:
-                return None
+        elif name in {
+            indicators.url.name,
+            indicators.socket.name,
+            indicators.hostname.name,
+            indicators.domain.name,
+            indicators.subdomain.name
+        }:
+            if self.args.filter >= 2:
+                if LetterWeights.IOC(value) < 0.6:
+                    return None
+                if name != indicators.url.name and len(value) > 0x100:
+                    return None
             ioc = value.decode(self.codec)
             if '://' not in ioc: ioc = F'tcp://{ioc}'
             parts = urlparse(ioc)
@@ -157,7 +216,7 @@ class xtp(PatternExtractor):
             for white, level in self._LEGITIMATE_HOSTS.items():
                 if self.args.filter >= level and hl == white or hl.endswith(F'.{white}'):
                     return None
-            if name == 'url':
+            if name == indicators.url.name:
                 scheme = parts.scheme.lower()
                 for p in ('http', 'https', 'ftp', 'file', 'mailto'):
                     if scheme.endswith(p):
@@ -166,7 +225,11 @@ class xtp(PatternExtractor):
                         break
             if any(hl == w for w in self._DOMAIN_WHITELIST):
                 return None
-            if name in {'hostname', 'domain', 'subdomain'}:
+            if name in {
+                indicators.hostname.name,
+                indicators.domain.name,
+                indicators.subdomain.name
+            }:
                 hostparts = host.split('.')
                 if self.args.filter >= 2:
                     # remove hostnames where no part is longer than three characters.
@@ -197,17 +260,35 @@ class xtp(PatternExtractor):
                             R'{}(?:\.\w+)+'.format(hostparts[0]).encode('ascii'), data)))
                         if seen_before > 2:
                             return None
-        elif name == 'email':
+        elif name == indicators.email.name:
             at = value.find(B'@')
             ix = 0
             while value[ix] not in self._ALPHABETIC:
                 ix += 1
             return None if at - ix < 3 else value[ix:]
-        elif name == 'path':
+        elif name == indicators.path.name:
             if len(value) < 8:
                 return None
             if len(value) > 16 and len(re.findall(RB'\\x\d\d', value)) > len(value) // 10:
                 return None
+            if self.args.filter >= 2:
+                try:
+                    path = value.decode(self.codec)
+                except Exception:
+                    return None
+                try:
+                    path = Path(path)
+                except Exception as E:
+                    self.log_debug(F'error parsing path "{path}": {E!s}')
+                    return None
+                for k, part in enumerate(path.parts):
+                    if not k:
+                        if part.endswith(':') and len(part) == 2:
+                            continue
+                        if part[0] == part[~0] == '%':
+                            continue
+                    if LetterWeights.Path(part) < 0.6:
+                        return None
         return value
 
     def process(self, data):
